package com.hexin.android.component.listview

import android.content.Context
import android.util.AttributeSet
import android.view.View
import androidx.core.widget.NestedScrollView

/**
 * 和SlidingRecyclerView配合的ScrollView
 * 使用该控件时需要设置以下内容：
 * 1、顶部视图的高度(topViewHeight)
 * 2、SlidingRecyclerView对象
 */
open class ListNestedScrollView(context: Context, attrs: AttributeSet) : NestedScrollView(context, attrs) {

    private var mStartFlingTime = 0L
    private var mVelocityY = 0f

    var topViewHeight = 0
    var slidingRecyclerView: SlidingRecyclerView? = null

    override fun onNestedPreScroll(target: View, dx: Int, dy: Int, consumed: IntArray, type: Int) {
        val hiddenTop = dy > 0 && scrollY < topViewHeight
        val showTop = dy < 0 && !target.canScrollVertically(-1)

        if (hiddenTop || showTop) {
            scrollBy(0, dy)
            consumed[1] = dy
        }
        if(scrollY == 0){
            slidingRecyclerView?.stopScroll()
        }
    }

    override fun onNestedPreFling(target: View, velocityX: Float, velocityY: Float): Boolean {
        mVelocityY = velocityY
        mStartFlingTime = System.currentTimeMillis()
        // 向上滑的时候需要拦截，先判断自己是否需要滑动
        if (velocityY > 0) {
            if (scrollY < topViewHeight) {
                fling(velocityY.toInt())
                return true
            }
        }
        return false
    }

    override fun onScrollChanged(l: Int, t: Int, oldl: Int, oldt: Int) {
        super.onScrollChanged(l, t, oldl, oldt)
        if (scrollY >= topViewHeight) { // 自己已经滑完了
            val timeDiff = System.currentTimeMillis() - mStartFlingTime
            val curVelocity = FeedflowNestedScrollHelper.getCurVelocityByPassedTime(mVelocityY.toDouble(), timeDiff)
            if (curVelocity <= 0) {
                return
            }
            slidingRecyclerView?.consumeRemaindVelocity(curVelocity.toInt())
        }
    }

    /**
     * 消耗剩余的速度
     */
    fun consumeRemaindVelocity() {
        if (mVelocityY >= 0) {
            return
        }
        val passedTime = System.currentTimeMillis() - mStartFlingTime
        val curVelocity = FeedflowNestedScrollHelper.getCurVelocityByPassedTime(mVelocityY.toDouble(), passedTime)
        if (curVelocity >= 0) {
            return
        }
        fling(curVelocity.toInt())
    }


    companion object {
        const val TAG = "ListNestedScrollView"
    }
}
